@@ -13,13 +13,9 @@
 
 #include "chan.h"
 
-#include <telepathy-glib/base-connection.h>
+#include <telepathy-glib/telepathy-glib.h>
 #include <telepathy-glib/channel-iface.h>
-#include <telepathy-glib/dbus.h>
-#include <telepathy-glib/gtypes.h>
-#include <telepathy-glib/interfaces.h>
 #include <telepathy-glib/svc-channel.h>
-#include <telepathy-glib/svc-generic.h>
 
 static void channel_iface_init (gpointer iface, gpointer data);
 static void destroyable_iface_init (gpointer iface, gpointer data);
@@ -134,7 +130,7 @@ send_message (GObject *object,
 
       /* in this example we ignore interface-specific parts */
 
-      s = tp_asv_get_string (input, "type");
+      s = tp_asv_get_string (input, "content-type");
 
       if (s == NULL)
         continue;
@@ -148,9 +144,9 @@ send_message (GObject *object,
 
       j = tp_message_append_part (received);
 
-      s = tp_asv_get_string (input, "type");
+      s = tp_asv_get_string (input, "content-type");
       g_assert (s != NULL);   /* already checked */
-      tp_message_set_string (received, j, "type", s);
+      tp_message_set_string (received, j, "content-type", s);
 
       s = tp_asv_get_string (input, "identifier");
 
