@@ -432,7 +432,12 @@ _tp_account_set_connection (TpAccount *account,
         }
     }
 
-  g_object_notify (G_OBJECT (account), "connection");
+  if (tp_strdiff (priv->connection_object_path, path))
+    {
+      g_free (priv->connection_object_path);
+      priv->connection_object_path = g_strdup (path);
+      g_object_notify (G_OBJECT (account), "connection");
+    }
 }
 
 static void
@@ -444,6 +449,27 @@ _tp_account_update (TpAccount *account,
   TpConnectionStatus old_s = priv->connection_status;
   gboolean presence_changed = FALSE;
 
+  if (g_hash_table_lookup (properties, "Interfaces") != NULL)
+    {
+      gchar **interfaces, **iter;
+
+      interfaces = tp_asv_get_boxed (properties, "Interfaces", G_TYPE_STRV);
+
+      for (iter = interfaces; *iter != NULL; iter++)
+        {
+          if (tp_dbus_check_valid_interface_name (*iter, NULL))
+            {
+              GQuark q = g_quark_from_string (*iter);
+
+              tp_proxy_add_interface_by_id ((TpProxy *) account, q);
+            }
+          else
+            {
+              DEBUG ("\t\tInterface %s not valid", *iter);
+            }
+        }
+    }
+
   if (g_hash_table_lookup (properties, "ConnectionStatus") != NULL)
     priv->connection_status =
       tp_asv_get_uint32 (properties, "ConnectionStatus", NULL);
@@ -580,16 +606,18 @@ _tp_account_update (TpAccount *account,
 
   if (g_hash_table_lookup (properties, "Connection") != NULL)
     {
-      g_free (priv->connection_object_path);
+      const gchar *path = tp_asv_get_object_path (properties, "Connection");
 
-      priv->connection_object_path =
-        g_strdup (tp_asv_get_object_path (properties, "Connection"));
-
-      if (priv->connection != NULL)
+      if (tp_strdiff (path, priv->connection_object_path))
         {
-          if (tp_strdiff (priv->connection_object_path,
-                  tp_proxy_get_object_path (priv->connection)))
+          g_free (priv->connection_object_path);
+
+          priv->connection_object_path = g_strdup (path);
+
+          if (priv->connection != NULL)
             _tp_account_free_connection (account);
+
+          g_object_notify (G_OBJECT (account), "connection");
         }
     }
 
@@ -845,6 +873,9 @@ _tp_account_finalize (GObject *object)
   g_array_free (priv->actual_features, TRUE);
   g_array_free (priv->missing_features, TRUE);
 
+  if (priv->parameters != NULL)
+    g_hash_table_unref (priv->parameters);
+
   /* free any data held directly by the object here */
   if (G_OBJECT_CLASS (tp_account_parent_class)->finalize != NULL)
     G_OBJECT_CLASS (tp_account_parent_class)->finalize (object);
@@ -2427,7 +2458,7 @@ tp_account_set_nickname_async (TpAccount *account,
   g_return_if_fail (nickname != NULL);
 
   result = g_simple_async_result_new (G_OBJECT (account),
-      callback, user_data, tp_account_request_presence_finish);
+      callback, user_data, tp_account_set_nickname_finish);
 
   if (nickname == NULL)
     {
