@@ -895,7 +895,8 @@ closure_invalidated_cb (gpointer ctx_,
  * @detailed_signal: a string of the form "signal-name::detail".
  * @c_handler: the #GCallback to connect.
  * @gobject: the object to pass as data to @c_handler.
- * @connect_flags: a combination of #GConnectFlags.
+ * @connect_flags: a combination of #GConnectFlags. Only
+ *  %G_CONNECT_AFTER and %G_CONNECT_SWAPPED are supported by this function.
  *
  * Connects a #GCallback function to a signal for a particular object, as if
  * with g_signal_connect(). Additionally, arranges for the signal handler to be
@@ -913,6 +914,8 @@ closure_invalidated_cb (gpointer ctx_,
  * user_data for callbacks without having to explicitly disconnect all the
  * handlers in their finalizers.
  *
+ * Changed in 0.10.4 and 0.11.3: %G_CONNECT_AFTER is now respected.
+ *
  * Returns: the handler id
  *
  * Since: 0.9.2
@@ -938,7 +941,7 @@ tp_g_signal_connect_object (gpointer instance,
     ctx->closure = g_cclosure_new_object (c_handler, gobject);
 
   ctx->handler_id = g_signal_connect_closure (instance, detailed_signal,
-      ctx->closure, 0);
+      ctx->closure, (connect_flags & G_CONNECT_AFTER) ? TRUE : FALSE);
 
   g_object_weak_ref (instance_obj, instance_destroyed_cb, ctx);
   g_object_weak_ref (gobject, observer_destroyed_cb, ctx);
@@ -1030,11 +1033,13 @@ tp_value_array_build (gsize length,
           g_free (error);
 
           g_value_array_free (arr);
+          va_end (var_args);
           return NULL;
         }
     }
 
   g_warn_if_fail (arr->n_values == length);
 
+  va_end (var_args);
   return arr;
 }
