@@ -4,18 +4,11 @@
 
 #include "tests/lib/myassert.h"
 
-static int fail = 0;
 static gboolean had_unsupported = FALSE;
 static gboolean had_supported = FALSE;
 static GMainLoop *mainloop = NULL;
 static gboolean freed_user_data[] = { FALSE, FALSE, FALSE, FALSE };
 
-static void
-myassert_failed (void)
-{
-  fail = 1;
-}
-
 static void
 supported_cb (TpDBusDaemon *bus_daemon,
               const gchar **names,
@@ -81,7 +74,8 @@ main (int argc,
   g_type_init ();
   tp_debug_set_flags ("all");
 
-  bus_daemon = tp_dbus_daemon_new (tp_get_bus ());
+  bus_daemon = tp_dbus_daemon_dup (NULL);
+  g_assert (bus_daemon != NULL);
 
   /* this interface is automatically supported... */
   MYASSERT (tp_cli_dbus_daemon_run_list_names (bus_daemon, -1, NULL,
@@ -126,5 +120,7 @@ main (int argc,
   MYASSERT (!freed_user_data[2], " (signal connection, supported)");
   MYASSERT (freed_user_data[3], " (signal connection, unsupported)");
 
-  return fail;
+  g_object_unref (bus_daemon);
+
+  return 0;
 }
